package Client;

import berechne.*;
import java.math.*;

public class Pi implements Aufgabe {

    /** Konstanten zur Berechnung von Pi*/
    private static final BigDecimal ZERO = BigDecimal.valueOf(0);
    private static final BigDecimal  ONE = BigDecimal.valueOf(1);
    private static final BigDecimal FOUR = BigDecimal.valueOf(4);

    /** Rundung bei der Berechnung festlegen */
    private static final int roundingMode = BigDecimal.ROUND_HALF_EVEN;

    /** Przision der Berechnung, nach dem Dezimalpunkt */
    private int digits;
    
    /**
     * Konstruktion einer Task, welche Pi mit der verlangten Genauigkeit berechnet
     */
    public Pi(int digits) {
        this.digits = digits;
        System.out.println("Pi: Konstruktor");
    }

    /**
     * Berechne pi.
     */
    public Object ausfhren() {
        System.out.println("Pi: Methode 'ausfhren()' ");
        return berechnePi(digits);
    }

    /**
     * Berechne den Wert von Pi mit der angegebenen Anzahl 
     * Stellen nach dem Dezimalpunkt.  Der Wert wird mit Hilfe der  
     * Machin's Formel berechnet:
     *
     *          pi/4 = 4*arctan(1/5) - arctan(1/239)
     *
     * und einer Reihenentwicklung von arctan(x) bis zur 
     * ausreichenden Genauigkeit.
     */
    public static BigDecimal berechnePi(int digits) {
        System.out.println("Pi: Methode 'berechnePi()' ");
        int scale = digits + 5;
        BigDecimal arctan1_5 = arctan(5, scale);
        BigDecimal arctan1_239 = arctan(239, scale);
        BigDecimal pi = arctan1_5.multiply(FOUR).subtract(
                                  arctan1_239).multiply(FOUR);
        return pi.setScale(digits, 
                           BigDecimal.ROUND_HALF_UP);
    }
    /**
     * Berechne den Wert, in Rad.Grad, des arctangent 
     * Der wert wird mit Hilfe der Reihenentwicklung berechnet
     *
     * arctan(x) = x - (x^3)/3 + (x^5)/5 - (x^7)/7 + 
     *     (x^9)/9 ...
     */   
    public static BigDecimal arctan(int inverseX, 
                                  int scale) 
    {
        System.out.println("Pi: Methode 'arctan()' ");
        BigDecimal result, numer, term;
        BigDecimal invX = BigDecimal.valueOf(inverseX);
        BigDecimal invX2 = 
            BigDecimal.valueOf(inverseX * inverseX);

        numer = ONE.divide(invX, scale, roundingMode);

        result = numer;
        int i = 1;
        do {
            numer = 
                numer.divide(invX2, scale, roundingMode);
            int denom = 2 * i + 1;
            term = 
                numer.divide(BigDecimal.valueOf(denom),
                             scale, roundingMode);
            if ((i % 2) != 0) {
                result = result.subtract(term);
            } else {
                result = result.add(term);
            }
            i++;
        } while (term.compareTo(ZERO) != 0);
        return result;
    }
}
